@@ -2,10 +2,9 @@
 #define __CR_PAGEMAP_H__
 
 #include <sys/types.h>
-#include "asm/page.h"
-#include "asm/int.h"
+#include "int.h"
 
-#include "list.h"
+#include "common/list.h"
 
 struct vma_area;
 
